//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.20
// Date modification : 08/01/2003
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


import java.util.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
public class Parcours {
  protected int nbrVilles ;
  protected int nbrVillesMax ;
  protected int[] villes ;
  protected double longueur = -1 ;
  // longueur du parcours
  protected double dist[][] ;
	
  //------------------------------
  public Parcours () {
    nbrVilles = 0 ; nbrVillesMax = 10 ; villes = new int[nbrVillesMax] ; dist = null ; }
  public Parcours (int n) {
    nbrVilles = n ; nbrVillesMax = n ; villes = new int[nbrVillesMax] ; dist = null ; }
  public Parcours (double d[][]) {
    nbrVilles = 0 ; nbrVillesMax = 10 ; villes = new int[nbrVillesMax] ; dist = d ; }
  public Parcours (int n, double d[][]) {
    nbrVilles = n ; nbrVillesMax = n ; villes = new int[nbrVillesMax] ; dist = d ; }
  public Parcours (Parcours p) {
    nbrVilles = p.nbrVilles ;
    nbrVillesMax = p.nbrVillesMax ;
    villes = new int[nbrVillesMax] ;
    for (int i=0;i<nbrVilles;i++) 
      villes[i] = p.villes[i] ;
    longueur = p.longueur ;
    dist = p.dist ; 
  }

  //------------------------------
  public double size () { return nbrVilles ; }
  public double getNbrVilles () { return nbrVilles ; }

  //------------------------------
  public int elementAt (int i) {
    if (i<nbrVilles) return villes[i] ;
    else return -1 ;
  }

  //------------------------------
  // Met l'lment j  l'indice i
  public int setElementAt (int i, int j) {
    if (i<nbrVilles) return villes[i]=j ;
    else return -1 ;
  }

  public int getVille (int i) { return elementAt(i) ; }

  //------------------------------
  public double getLongueur () {
    if (longueur>=0) return longueur ;
    return calculeLongueur() ;
  }

  //------------------------------
  // Augmente la dimension du tableau villes
  public void realloueVilles () {
    int nbrVillesMax2 = nbrVilles + 20 ;
    int [] villes2 = new int[nbrVillesMax2] ;
    for (int i=0;i<nbrVillesMax;i++)
      villes2[i] = villes[i] ;
    villes = villes2 ;
    nbrVillesMax = nbrVillesMax2 ;
  }
  
  //------------------------------
  // Ajoute la ville i dans le parcours si possible
  // et augmente le nombre de villes du parcours !!!
  public boolean add (int ville) {
    longueur = -1 ;
    for (int i=0;i<nbrVilles;i++) {
      if (villes[i]==ville) return false ;
    }
    if (nbrVilles>=nbrVillesMax) {
      realloueVilles() ;
    }
    villes[nbrVilles] = ville ;
    nbrVilles++ ;
    return true ;
  }

  //------------------------------
  //	public boolean add (int indice, int i) {
  //		for (int j=0;j<indice;j++) {
  //			// si l'element existe => on renvoie faux
  //			if ( ((Integer)villes.elementAt(j)).intValue() == i ) return false ; 
  //		}
  //		villes.add (indice, new Integer(i)) ;
  //		return true ;
  //	}

  //------------------------------
  // Calcul la longueur d'un parcours (une solution du TSP)
  public double calculeLongueur()
  {
    if (dist==null) return -1 ;

    longueur = 0 ;
    if (nbrVilles==0) return 0 ;

    for (int i = 0 ; i <nbrVilles-1 ; i++) {
      int j = villes[i] ;
      int k = villes[i+1] ;
      longueur = longueur + dist[j][k] ;
    }
    int j = villes[nbrVilles-1] ;
    int k = villes[0] ;
    longueur = longueur + dist[j][k] ;
    return longueur ;
  }

  //------------------------------
  // Calcul la longueur d'un parcours (une solution du TSP)
  public double calculeLongueur(double d[][])
  {
    dist = d ;
    return calculeLongueur() ;
  }
    	
  //------------------------------
  // generation aleatoire d'un parcours
  public void genereAleat(Random rand)
  {
    longueur = -1 ;
    int i = 0 ;
    while (i<nbrVilles) {
      int k = rand.nextInt(nbrVilles) ;
      boolean trouve = false ;
      for (int j=0;j<i;j++)
        if (villes[j]==k) trouve = true ;
      if (!trouve) {
        setElementAt(i, k) ;
        i++ ;
        
      }
    }
  }
	
  //------------------------------
  // generation aleatoire d'un parcours
  public void genereAleat()
  {
    genereAleat (new Random(new Date().getTime())) ;
  }
	
  //------------------------------
  // generation d'un parcours glouton
  public void genereGlouton()
  {
    int[] villes_candidates = new int[nbrVilles] ;
    for (int i=0;i<nbrVilles;i++)
      villes_candidates[i] = i ;
    
    int i = 1 ;
    villes[0] = 0 ;
    villes_candidates[0] = -1 ;
    while (i<nbrVilles) {
      double longmin = -1 ;
      int new_ville = -1 ;
      
      for (int j=0;j<nbrVilles;j++) {
        if (villes_candidates[j]>=0) {
          double longueur = dist[villes[i-1]][j] ;
      
          // calcul dist entre ville villes[i-1] et ville j
          if ((longmin<0) || (longueur<longmin)) {
            longmin = longueur ;
            new_ville = j ;
          }
        }
      }
      villes[i] = new_ville ;
      i++ ;
      villes_candidates[new_ville] = -1 ;
    }
  }
  
	
  //------------------------------
  /**
   * generation alatoire d'un voisin 2-opt :
   * on tire un indice ind1 < ind2 et on cree un nouveau
   * parcours constitue des villes du parcours initial :
   * - de 0  ind1
   * - de ind2  ind1+1 (en dcroissant)
   * - de ind2+1  nbrVilles-1
   */
  Parcours genereVoisin2Opt(Random rand)
  {
    Parcours p = new Parcours (this) ;
    int ind2 = rand.nextInt(nbrVilles-3)+3 ;
    int ind1 = rand.nextInt(ind2-1) ;
    
    for (int j=ind1+1;j<=ind2;j++)
      p.setElementAt(j, this.elementAt(ind2-(j-(ind1+1)))) ;

    return p ;
  }
    
  //------------------------------
  // convertion en String
  public String toString()
  {
    String st = new String() ;
    for (int j=0;j<nbrVilles;j++) {
      st = st + villes[j] ; 
      if (j<nbrVilles-1) st = st + "/" ; 
    }
    st = st + " : " + longueur ;
    return st ;
  }

}



