//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.20
// Date modification : 08/01/2003
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import java.util.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Voyageur de commerce
public class Tsp {
  public static final boolean debug = true ;

  // TSPLIB
  String namePb, nameSol ;
  String commentPb ;
  String typePb, typeSol ;
  int dimensionPb ; // Nbr de villes
  String edgeWeightTypePb ;

  // Liste des coordonnees des villes et matrice des distance
  private Vector listePts ;
  private double dist[][] ;

  // TSP
  Parcours solOpt ; // Meilleure solution obtenue
  Parcours solOptTh ; // Solution optimale connue
  long nbrSol ;

  //------------------------------
  public Tsp() { listePts = new Vector() ; } 

  //------------------------------
  public Tsp(String stPb, String stSol) { 
    lireFichierProbleme (stPb) ; 
    lireFichierSolution (stSol) ; 
  } 

  //------------------------------
  public int size() { return listePts.size() ; }
  public int getNbrPoints() { return listePts.size() ; }
  public Point2D getPoint(int i) { return (Point2D)listePts.get(i) ; }

  public double [][] getDist() { return dist ; }
  public int getSize() { return dimensionPb ; }

  //------------------------------
  public void lireFichierProbleme (String chaine) {
    if (chaine==null) return ;

    /*
    System.out.println ("Lecture du fichier probleme : "+chaine) ;
    */
		
    String delim = new String(":") ;
    int nodeCoordSection = 0, encore = 1 ;
    FichierEntree fe = new FichierEntree (chaine) ;

    listePts = new Vector() ;
    while (encore == 1) {
      String s = fe.lireLigne() ;
      if (nodeCoordSection==0) {
        StringTokenizer st = new StringTokenizer(s, delim) ;
        String sg = st.nextToken().trim() ;
        if (sg.compareToIgnoreCase("NAME")==0) namePb = new String (st.nextToken().trim()) ;			 
        if (sg.compareToIgnoreCase("COMMENT")==0) commentPb = new String (st.nextToken().trim()) ;
        if (sg.compareToIgnoreCase("TYPE")==0) typePb = new String (st.nextToken().trim()) ;
        if (sg.compareToIgnoreCase("EDGE_WEIGHT_TYPE")==0) edgeWeightTypePb = new String (st.nextToken().trim()) ;
        if (sg.compareToIgnoreCase("DIMENSION")==0) {
          dimensionPb = Integer.parseInt(st.nextToken().trim()) ;
        }	
        if (sg.compareToIgnoreCase("NODE_COORD_SECTION")==0) nodeCoordSection = 1 ;
      }
      else {
        StringTokenizer st = new StringTokenizer(s) ;
        int num = (new Double(st.nextToken())).intValue() ;
        double x = (new Double(st.nextToken())).doubleValue() ;
        double y = (new Double(st.nextToken())).doubleValue() ;
        listePts.add (new Point2D(x,y)) ;
        if (listePts.size()>=dimensionPb) encore = 0 ;
      }
    }
    fe.fermer() ;

    calculerDist() ;
  }

  //------------------------------
  public void lireFichierSolution (String chaine) {
    if (chaine==null) return ;

    /*
    System.out.println ("Lecture du fichier solution : "+chaine) ;
    */

    String delim = new String(":") ;
    int tourSection = 0, encore = 1 ;
    FichierEntree fe = new FichierEntree (chaine) ;

    solOptTh = new Parcours() ;
    while (encore == 1) {
      String s = fe.lireLigne() ;
      if (tourSection==0) {
        StringTokenizer st = new StringTokenizer(s, delim) ;
        String sg = st.nextToken().trim() ;
        if (sg.compareToIgnoreCase("NAME")==0) nameSol = new String (st.nextToken().trim()) ;			 
        if (sg.compareToIgnoreCase("TYPE")==0) typeSol = new String (st.nextToken().trim()) ;
        if (sg.compareToIgnoreCase("DIMENSION")==0) {
          if (Integer.parseInt(st.nextToken().trim())!=dimensionPb) encore = 0 ;
        }	
        if (sg.compareToIgnoreCase("TOUR_SECTION")==0) tourSection = 1 ;
      }
      else {
        StringTokenizer st = new StringTokenizer(s) ;
        int num = (new Double(st.nextToken())).intValue() ;
        solOptTh.add (num-1) ;
        if (solOptTh.size()>=dimensionPb) encore = 0 ;
      }
    }
    fe.fermer() ;
    if (solOptTh!=null) solOptTh.calculeLongueur(dist) ;
    solOpt = solOptTh ;
  }

  //------------------------------
  // Calcul de la matrice des distances
  public void calculerDist()
  {
    dist = new double[dimensionPb][dimensionPb] ;
    for (int i=0;i<dimensionPb;i++) {
      for (int j=0;j<dimensionPb;j++) {
        Point2D p1 = (Point2D)listePts.get(i) ;
        Point2D p2 = (Point2D)listePts.get(j) ;
        double xd = (double)p1.getX() - (double)p2.getX() ;
        double yd = (double)p1.getY() - (double)p2.getY() ;
        dist[i][j] = Math.sqrt (xd*xd+yd*yd) ;
      }
    }

  }

  //------------------------------
  // Renvoie la dimension du rectangle englobant des villes
  public Point2D getPoint2DMax()
  {
    int xmax=0, ymax=0 ;
    for (int i=0;i<dimensionPb;i++) {
      Point2D p1 = (Point2D)listePts.get(i) ;
      if (p1.getX()>xmax) xmax = (int)p1.getX() ;
      if (p1.getY()>ymax) ymax = (int)p1.getY() ;
    }
    return (new Point2D (xmax, ymax)) ;

  }

  //------------------------------
  // Affiche les paramtres du probleme TSP a l'ecran
  public void afficherEcran()
  {
    System.out.println ("NAME : "+namePb) ;
    System.out.println ("COMMENT : "+commentPb) ;
    System.out.println ("TYPE : "+typePb) ;
    System.out.println ("DIMENSION : "+dimensionPb) ;
    System.out.println ("EDGE_WEIGHT_TYPE : "+edgeWeightTypePb) ;
    System.out.println ("NODE_COORD_SECTION : ") ;
    for (int i = 1 ; i <= dimensionPb ; i++) {
      Point2D p = (Point2D)listePts.get(i-1) ;
      System.out.print("("+p.getX()+","+p.getY()+"),") ;
    }
    System.out.println() ;
    if (solOptTh!=null) {
      System.out.println("Solution optimale theorique : "+solOptTh.toString()) ;
    }
  }

  //------------------------------
  public static void main(String args[]) {
    String nomPb=null, nomSol=null ;
    if ((args.length>=1) && (args.length<=2)) {
      if (args.length==2) nomSol = new String (args[1]) ;
      if (args.length>=1) nomPb = new String (args[0]) ;
      Tsp tsp = new Tsp(nomPb, nomSol) ;
      tsp.afficherEcran() ;
    }
    else
      System.out.println ("Usage : java Tsp nomProblem [nomSolutionOptimale]") ;
  }
}
