//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.31
// Date modification : 08/01/2003
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import java.awt.*;
import java.awt.event.* ;
import javax.swing.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
public class TspJFrame extends JFrame implements Runnable {
  private boolean debug = Tsp.debug ;

  private Tsp tsp ;
  private TspSolver tspSolver ;
  
  private Thread thread, threadPerf ;
  
  private boolean finished=true, exitRun=true ;
  
  private TspJPanel tspJPanel ;
  private JFrame tspJFramePerf ;
  private JScrollPane jScrollPane ;
  private JLabel jLabelLongueur ;

  //--------------------------------
  public TspJFrame(Tsp t) { 
    tsp = t ;
	
    // Nouveau panneau a barre de defilement pour afficher les villes
    tspJPanel = new TspJPanel (tsp) ;
    JScrollPane jScrollPane = new JScrollPane (tspJPanel ,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS) ;
    	
    // de taille graphique 500 pixels sur 500 pixels
    jScrollPane.setPreferredSize(new Dimension(500,500)); 
        
    // Taille totale de la zone d'affichage en utilisant les barres de defilement
    tspJPanel.setJScrollPane(jScrollPane) ;
    tspJPanel.calculerZoomVueDEnsemble() ;
    	
    // Nouveau panneau pour afficher des informations (longueur du parcours)
    JPanel jp2 = new JPanel() ;
    jLabelLongueur = new JLabel("Longueur : ") ;
    jp2.add (jLabelLongueur) ;
    miseAJourInformation() ;
		
    // Titre de la fenetre
    setTitle("Travelling Salesman Problem : "+tsp.namePb)  ;
	
    // Implantation de type Border
    getContentPane().setLayout(new BorderLayout());
       	
    // Ajout du panneau a barre de defilement au centre
    getContentPane().add(jScrollPane, BorderLayout.CENTER); 
	
    // Ajout du panneau d'information en bas
    getContentPane().add (jp2, BorderLayout.SOUTH) ;
	
    // Traitement de la fermeture de fenetre
    addWindowListener (new AppCloser()) ;
	
    // Menu
    setJMenuBar (tspJMenuBar()) ;
	
    pack();
    show();
    tspJPanel.calculerZoomVueDEnsemble() ;
    tspJPanel.repaint() ;

  } 

  //--------------------------------
  // mise a jour des informations
  public void miseAJourInformation() { 
    String st = new String () ;
    if (tspSolver!=null) 	st = st + "Iteration : " + tspSolver.getIter() + " (" + tspSolver.getDuree() + ") - ";
    st = st + "Longueur = " ;
    if (tsp.solOpt!=null) 	st = st + tsp.solOpt.getLongueur() ;
    else			st = st + "?" ;
    st = st + " - Longueur optimale = " ;
    if (tsp.solOptTh!=null) st = st + tsp.solOptTh.getLongueur() ;
    else {  // on affiche la meilleure solution trouve jusque la
      if (tsp.solOpt!=null)
        st = st + "#" + tsp.solOpt.getLongueur() ;
      else
        st = st + "?" ;
    }
    jLabelLongueur.setText (st) ;
  }

  //--------------------------------
  public void startSolver(TspSolver s) { 
    if (debug) System.out.println ("TspJFrame startSolver : ") ;
    tspSolver = s ;
    synchronized(this) {
      while (!exitRun) {
        if (debug) System.out.println ("TspJFrame startSolver : Waiting for Solver terminates... ") ;
        try{wait();}catch(Exception e){}
      }
    }
    thread = new Thread(this) ;
    thread.start() ;
    if (debug) System.out.println ("TspJFrame startSolver : thread.start()") ;
  }

  //--------------------------------
  public void stopSolver() { 
    if (debug) System.out.println ("TspJFrame stopSolver") ;
    if (thread!=null) {
      finished = true ;
    }
    // Suppression de la fenetre
    if (tspJFramePerf!=null) {
      tspJFramePerf.hide() ;
      tspJFramePerf= null ;
    }
  }

  //--------------------------------
  // methode run du thread
  public void run() { 
    finished = false ;
    exitRun = false ;
    while (!finished) {
      
      switch (tspSolver.run()) {
      case TspSolver.RAFRAICHIR_TOUT : // tout rafraichir
        repaint() ; 
        miseAJourInformation() ;
        break ;
      case TspSolver.RAFRAICHIR_ITER : // rafraichir l'iteration
        miseAJourInformation() ;
        break ;
      case TspSolver.STOP : //stop
        repaint() ; 
        miseAJourInformation() ;
        stopSolver() ;
        break ;
      }
    }
        
    exitRun = true ;

    if (debug) {
      System.out.print("TspJFrame Run : exit - ") ;
      System.out.println ("finished="+finished+" - exitRun="+exitRun) ;
    }
    synchronized(this){notify() ;}
  }

  //--------------------------------
  // creation de la barre de menu
  public JMenuBar tspJMenuBar() { 
    JMenuBar jmb = new JMenuBar() ;
	
    //------
    // Menu Fichier
    JMenu jmFic = new JMenu("Fichier") ;
    jmb.add (jmFic) ;
	
    // Menu Fichier - Item Ouvrir
    JMenuItem jmiFic1 = new JMenuItem ("Ouvrir") ;
    jmiFic1.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuFichierItemOuvrir(evt); } } );	
    jmFic.add (jmiFic1) ;
        
    // Menu Fichier - Item Quitter
    JMenuItem jmiFic2 = new JMenuItem ("Quitter") ;
    jmiFic2.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuFichierItemQuitter(evt); } } );	
    jmFic.add (jmiFic2) ;
        
    //------
    // Menu TSP Exact
    JMenu jmTspExact = new JMenu("Mthodes Exactes") ;
    jmb.add (jmTspExact) ;
	
    // Menu TSP Exact - Item Solveur Little Standard
    JMenuItem jmiTsp1ls = new JMenuItem ("Algorithme de Little standard") ;
    jmiTsp1ls.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuTspItemSolveurLittleStandard(evt); } } );	
    jmTspExact.add (jmiTsp1ls) ;
        
    // Menu TSP Exact - Item Solveur Little Amlior
    JMenuItem jmiTsp1la = new JMenuItem ("Algorithme de Little Amlior") ;
    jmiTsp1la.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuTspItemSolveurLittleAmeliore(evt); } } );	
    jmTspExact.add (jmiTsp1la) ;
        
    //------
    // Menu TSP Approch
    JMenu jmTsp = new JMenu("Mthodes Approches") ;
    jmb.add (jmTsp) ;
	
    // Menu TSP - Item Une solution aleatoire
    JMenuItem jmiTsp1 = new JMenuItem ("Une solution aleatoire") ;
    jmiTsp1.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuTspItemUneSolutionAleatoire(evt); } } );	
    jmTsp.add (jmiTsp1) ;
        
    // Menu TSP - Item solveur aleatoire
    JMenuItem jmiTsp2 = new JMenuItem ("Solveur aleatoire") ;
    jmiTsp2.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuTspItemSolveurAleatoire(evt); } } );	
    jmTsp.add (jmiTsp2) ;

    // Menu TSP - Item Solveur Recuit
    JMenuItem jmiTsp2r = new JMenuItem ("Solveur Recuit Simul") ;
    jmiTsp2r.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuTspItemSolveurRecuit(evt); } } );	
    jmTsp.add (jmiTsp2r) ;
        
    // Menu TSP - Item Solveur Tabou
    JMenuItem jmiTsp2t = new JMenuItem ("Solveur Tabou") ;
    jmiTsp2t.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuTspItemSolveurTabou(evt); } } );	
    jmTsp.add (jmiTsp2t) ;
        
    // Menu TSP - Item solveur Algo Genetique
    JMenuItem jmiTsp3 = new JMenuItem ("Solveur Algorithme Genetique") ;
    jmiTsp3.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuTspItemSolveurAlgoGenetique(evt); } } );	
    jmTsp.add (jmiTsp3) ;
        
    // Menu TSP - Item solveur Ant
    JMenuItem jmiTsp4 = new JMenuItem ("Solveur Colonie de Fourmis") ;
    jmiTsp4.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuTspItemSolveurAnt(evt); } } );	
    jmTsp.add (jmiTsp4) ;
        
    // Menu TSP - Item STOP solveur
    JMenuItem jmiTsp5 = new JMenuItem ("STOP Solveur") ;
    jmiTsp5.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { stopSolver(); } } );	
    jmTsp.add (jmiTsp5) ;
        
    //------
    // Menu Affichage
    JMenu jmAff = new JMenu("Affichage") ;
    jmb.add (jmAff) ;
	
    // Menu Affichage - Item Vue d'ensemble
    JMenuItem jmAff1 = new JMenuItem ("Zoom Normal (100%)") ;
    jmAff1.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuAffichageItemZoomNormal(evt); } } );	
    jmAff.add (jmAff1) ;
        
    // Menu Affichage - Item Vue d'ensemble
    JMenuItem jmAff2 = new JMenuItem ("Vue d'ensemble") ;
    jmAff2.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuAffichageItemVueDEnsemble(evt); } } );	
    jmAff.add (jmAff2) ;
        
    //------
    // Menu Aide
    JMenu jm3 = new JMenu("Aide") ;
    jmb.add (jm3) ;
	
    // Menu Aide - Item A propos
    JMenuItem jmi31 = new JMenuItem ("A propos") ;
    jmi31.addMouseListener(new MouseAdapter() { public void mouseReleased(MouseEvent evt) { menuAideItemAPropos(evt); } } );	
    jm3.add (jmi31) ;
        
    return jmb;
  }
    
  //--------------------------------
  void menuFichierItemOuvrir(MouseEvent evt)
  {
    // Etape 1 : Ouvrir un probleme de TSP
    JFileChooser chooser = new JFileChooser(".");
    chooser.setDialogTitle("Ouvrir un probleme de TSP")  ;
	
    // Note: source for ExampleFileFilter can be found in FileChooserDemo,
    // under the demo/jfc directory in the Java 2 SDK, Standard Edition.
    int returnVal = chooser.showOpenDialog(this);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      System.out.println("Ouvrir un probleme de TSP : " + chooser.getSelectedFile().getAbsolutePath());
      tsp = new Tsp() ;
      tsp.lireFichierProbleme(chooser.getSelectedFile().getAbsolutePath()) ;
      setTitle("Travelling Salesman Problem : "+tsp.namePb)  ;
      tspJPanel.setTsp(tsp) ;
      tspJPanel.calculerZoomVueDEnsemble() ;
      repaint() ;
		
      // Etape 2 : Ouvrir la solution optimale
      JFileChooser chooser2 = new JFileChooser(".");
      chooser2.setDialogTitle("Ouvrir la solution optimale")  ;
	
      int returnVal2 = chooser2.showOpenDialog(this);
      if(returnVal2 == JFileChooser.APPROVE_OPTION) {
        System.out.println("Ouvrir la solution optimale : " + chooser2.getSelectedFile().getAbsolutePath());
        tsp.lireFichierSolution(chooser2.getSelectedFile().getAbsolutePath()) ;
        repaint() ;
      }
    }
    miseAJourInformation() ;
  }

  //--------------------------------
  void menuFichierItemQuitter(MouseEvent evt) 
  { 
    System.exit(0); 
  }

  //--------------------------------
  void menuTspItemUneSolutionAleatoire(MouseEvent evt) 
  { 
    if (tsp==null) return ;
    tsp.solOpt = new Parcours (tsp.size(), tsp.getDist()) ;
    tsp.solOpt.genereAleat() ;
    tsp.solOpt.calculeLongueur() ;
    tspJPanel.calculerZoomVueDEnsemble(); 
    repaint() ;
    miseAJourInformation() ;
  }

  //--------------------------------
  void menuTspItemSolveurAleatoire(MouseEvent evt) 
  { 
    if (tsp==null) return ;
    stopSolver() ;
    startSolver(new TspSolverAleat(tsp)) ;
  }

  //--------------------------------
  void menuTspItemSolveurLittleStandard(MouseEvent evt) 
  { 
    if (tsp==null) return ;
    stopSolver() ;

    TspSolverLittle sl = new TspSolverLittle(tsp, TspSolverLittle.STANDARD) ;

    startSolver(sl) ;
  }

  //--------------------------------
  void menuTspItemSolveurLittleAmeliore(MouseEvent evt) 
  { 
    if (tsp==null) return ;
    stopSolver() ;

    TspSolverLittle sl = new TspSolverLittle(tsp, TspSolverLittle.AMELIORE) ;

    startSolver(sl) ;
  }

  //--------------------------------
  void demanderParamSolveurRecuit (final TspSolverRecuit sr) 
  { 
    final JDialog f = new JDialog(this, "Paramtres Solveur Recuit") ;
    final JTextField jtf1, jtf2, jtf3 ;
    JButton jb ;
    f.getContentPane().setLayout(new GridLayout(4,2));
    f.getContentPane().add (new JLabel(" Temperature initiale ")) ;
    f.getContentPane().add (jtf1 = new JTextField("100")) ;
    f.getContentPane().add (new JLabel(" Coefficient refroidissement ")) ;
    f.getContentPane().add (jtf2 = new JTextField("0.9")) ;
    f.getContentPane().add (new JLabel(" Taille des paliers ")) ;
    f.getContentPane().add (jtf3 = new JTextField("10000")) ;
    f.getContentPane().add (jb = new JButton("Valider")) ;
    jb.addActionListener (new ActionListener() {public void actionPerformed(ActionEvent e) { 
      sr.modifParam(jtf1.getText(), jtf2.getText(), jtf3.getText()) ; 
      f.dispose() ; } } ) ;

    f.pack() ;
    f.setLocation(200,200) ;
    f.show() ;

  }

  //--------------------------------
  void menuTspItemSolveurRecuit(MouseEvent evt) 
  { 
    if (tsp==null) return ;
    stopSolver() ;

    TspSolverRecuit sr = new TspSolverRecuit(tsp) ;
    demanderParamSolveurRecuit (sr) ;

    startSolver(sr) ;
  }

  //--------------------------------
  void demanderParamSolveurTabou (final TspSolverTabou st) 
  { 
    final JDialog f = new JDialog(this, "Paramtres Solveur Recuit") ;
    final JTextField jtf1, jtf2 ;
    JButton jb ;
    f.getContentPane().setLayout(new GridLayout(3,2));
    f.getContentPane().add (new JLabel(" Longueur Liste Tabou ")) ;
    f.getContentPane().add (jtf1 = new JTextField("10")) ;
    f.getContentPane().add (new JLabel(" Critre d'aspiration ")) ;
    f.getContentPane().add (jtf2 = new JTextField("0.8")) ;
    f.getContentPane().add (jb = new JButton("Valider")) ;
    jb.addActionListener (new ActionListener() {public void actionPerformed(ActionEvent e) { 
      st.modifParam(jtf1.getText(), jtf2.getText()) ; 
      f.dispose() ; } } ) ;

    f.pack() ;
    f.setLocation(200,200) ;
    f.show() ;

  }

  //--------------------------------
  void menuTspItemSolveurTabou(MouseEvent evt) 
  { 
    if (tsp==null) return ;
    stopSolver() ;

    TspSolverTabou st = new TspSolverTabou(tsp) ;
    demanderParamSolveurTabou (st) ;

    startSolver(st) ;
  }

  //--------------------------------
  void demanderParamSolveurAG (final TspSolverAG sag) 
  { 
    final JDialog f = new JDialog(this, "Paramtres Solveur AG") ;
    final JTextField jtf1, jtf2, jtf3, jtf4 ;
    JButton jb ;
    f.getContentPane().setLayout(new GridLayout(5,2));
    f.getContentPane().add (new JLabel(" Taille population ")) ;
    f.getContentPane().add (jtf1 = new JTextField("1000")) ;
    f.getContentPane().add (new JLabel(" Nombre gnration ")) ;
    f.getContentPane().add (jtf2 = new JTextField("1000")) ;
    f.getContentPane().add (new JLabel(" Coeff selection % ")) ;
    f.getContentPane().add (jtf3 = new JTextField("5")) ;
    f.getContentPane().add (new JLabel(" Taux mutation   % ")) ;
    f.getContentPane().add (jtf4 = new JTextField("1")) ;
    f.getContentPane().add (jb = new JButton("Valider")) ;
    jb.addActionListener (new ActionListener() {public void actionPerformed(ActionEvent e) { sag.modifParam(jtf1.getText(), jtf2.getText(), jtf3.getText(),jtf4.getText()) ; f.dispose() ;} } ) ;

    f.pack() ;
    f.setLocation(200,200) ;
    f.show() ;
  }

  //--------------------------------
  void menuTspItemSolveurAlgoGenetique(MouseEvent evt) 
  { 
    if (tsp==null) return ;
    stopSolver() ;

    TspSolverAG sag = new TspSolverAG(tsp) ;
    demanderParamSolveurAG (sag) ;

    startSolver(sag) ;

    tspJFramePerf = new JFrame("AG Best Longueur Cycle") ;
    TspJPanelPerf tspJPanelPerf = new TspJPanelPerf(tspJFramePerf, sag, tsp) ;
    tspJFramePerf.getContentPane().add (tspJPanelPerf) ;
    tspJFramePerf.setSize(300,100) ;
    tspJFramePerf.setLocation(200,400) ;
    tspJFramePerf.show() ;
    threadPerf = new Thread (tspJPanelPerf) ;
    tspJPanelPerf.setThread (threadPerf) ;
    threadPerf.start() ;	
  }

  //--------------------------------
  void demanderParamSolveurAnt (final TspSolverAnt sant) 
  { 
    final JDialog f = new JDialog(this, "Paramtres Solveur Ant") ;
    final JTextField jtf1, jtf2, jtf3, jtf4 ;
    JButton jb ;
    f.getContentPane().setLayout(new GridLayout(5,2));
    f.getContentPane().add (new JLabel(" Nombre d'itrations ")) ;
    f.getContentPane().add (jtf1 = new JTextField("10000")) ;
    f.getContentPane().add (new JLabel(" Alpha               ")) ;
    f.getContentPane().add (jtf2 = new JTextField("1.0")) ;
    f.getContentPane().add (new JLabel(" Beta                ")) ;
    f.getContentPane().add (jtf3 = new JTextField("5.0")) ;
    f.getContentPane().add (new JLabel(" Ro                  ")) ;
    f.getContentPane().add (jtf4 = new JTextField("0.5")) ;
    f.getContentPane().add (jb = new JButton("Valider")) ;
    jb.addActionListener (new ActionListener() {public void actionPerformed(ActionEvent e) { sant.modifParam(jtf1.getText(), jtf2.getText(), jtf3.getText(),jtf4.getText()) ; f.dispose() ;} } ) ;

    f.pack() ;
    f.setLocation(200,200) ;
    f.show() ;
  }

  //--------------------------------
  void menuTspItemSolveurAnt(MouseEvent evt) 
  { 
    if (tsp==null) return ;
    stopSolver() ;

    TspSolverAnt sant = new TspSolverAnt(tsp) ;
    demanderParamSolveurAnt (sant) ;

    startSolver(sant) ;

    tspJFramePerf = new JFrame("ACO Best Longueur Cycle") ;
    TspJPanelPerf tspJPanelPerf = new TspJPanelPerf(tspJFramePerf, sant, tsp) ;
    tspJFramePerf.getContentPane().add (tspJPanelPerf) ;
    tspJFramePerf.setSize(300,100) ;
    tspJFramePerf.setLocation(200,400) ;
    tspJFramePerf.show() ;
    threadPerf = new Thread (tspJPanelPerf) ;
    tspJPanelPerf.setThread (threadPerf) ;
    threadPerf.start() ;	
  }

  //--------------------------------
  void menuAffichageItemZoomNormal(MouseEvent evt) 
  { 
    tspJPanel.calculerZoomNormal();
    repaint() ;
  }

  //--------------------------------
  void menuAffichageItemVueDEnsemble(MouseEvent evt) 
  { 
    tspJPanel.calculerZoomVueDEnsemble(); 
    repaint() ;
  }

  //--------------------------------
  void menuAideItemAPropos (MouseEvent evt) 
  { 
      System.out.println("===> A PROPOS");
      JFrame f = new JFrame("A propos") ;
      f.getContentPane().setLayout(new GridLayout(4,1));
      f.getContentPane().add (new JLabel(" Logiciel : gtsp ")) ;
      f.getContentPane().add (new JLabel(" Version : 0.31")) ;
      f.getContentPane().add (new JLabel(" Date : 08/01/2003")) ;
      f.getContentPane().add (new JLabel(" Auteur : Olivier Grunder ")) ;
      f.pack() ;
      f.show() ;
  }

  /**
   * To shutdown when run as an application.  This is a
   * fairly lame implementation.   A more self-respecting
   * implementation would at least check to see if a save
   * was needed.
   */
  protected static final class AppCloser extends WindowAdapter {
    public void windowClosing(WindowEvent e) {
      System.exit(0);
    }
  }
}


