//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.31
// Date modification : 08/01/2003
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


import java.awt.*;
import java.awt.event.* ;
import javax.swing.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
public class TspJPanel extends JPanel {
    private Tsp tsp ;
    
    private static int radius = 2 ;
    private double facteurZoom = 1.0 ;
    
    private JScrollPane jScrollPane ;
    
    //--------------------------------
    public TspJPanel() { tsp = null ; } 
    public TspJPanel(Tsp t) { tsp = t ; } 

    public void setJScrollPane(JScrollPane jsp) { jScrollPane = jsp ; }
    public void setTsp(Tsp t) { tsp = t ; }

    // ----------------------------------------------------------
    synchronized public void paint(Graphics g) {
	// Efface la zone d'affichage
        g.setColor(Color.lightGray);
	g.fillRect(0, 0, getSize().width - 1, getSize().height - 1);
        g.setColor(Color.black);
	g.drawRect(0, 0, getSize().width - 1, getSize().height - 1);
	
	// On s'arrete si le pb tsp n'existe pas
	if (tsp==null) return ;

	// Dessine le parcours si il existe
        Parcours parcours = tsp.solOpt ; 
	if (parcours!=null) {
        	g.setColor(Color.blue);
		int ville1, ville2 = parcours.getVille(0) ;
		Point2D pt2d1, pt2d2 = tsp.getPoint(ville2) ;
		for (int i=0;i<parcours.size();i++) {
			ville1 = ville2 ;
			if (i<parcours.size()-1) ville2 = parcours.getVille(i+1) ;
			else ville2 = parcours.getVille(0) ;
			pt2d1 = pt2d2 ;
			pt2d2 = tsp.getPoint(ville2) ;
		        g.drawLine ((int)(pt2d1.getX()*facteurZoom), (int)(pt2d1.getY()*facteurZoom), 
		        	(int)(pt2d2.getX()*facteurZoom), (int)(pt2d2.getY()*facteurZoom));
		}
	}

	// Dessine les villes toutes de la meme taille quelque soit le zoom
        g.setColor(Color.yellow);
        double rad = (double)radius, rad2 = (double)(2*radius) ;

	for (int i=0;i<tsp.getNbrPoints();i++) {
		Point2D pt2d = tsp.getPoint(i) ;
		int x = (int)pt2d.getX() ;
		int y = (int)pt2d.getY() ;
		
	        g.fillRect ((int)(x*facteurZoom-rad), (int)(y*facteurZoom-rad), (int)(rad2), (int)(rad2));
	}

    }

    //--------------------------------
    // calcule du facteur de zoom pour voir toutes les villes dans la zone d'affichage
    public void calculerZoomNormal() { 
	Point2D pt2d ;
    	if (tsp!=null) pt2d = tsp.getPoint2DMax() ;
    	else pt2d = new Point2D (500,500) ;
    	
    	facteurZoom = 1 ;
    	
    	// Le viewport est mis a la taille reelle du graphe eventuellement plus grand que les dimensions
    	// graphiques du JScrollPane, ce qui necessitera l'utilisation des scrollers pour visualiser
    	// le graphe entier.
        setPreferredSize(new Dimension((int)pt2d.getX(), (int)pt2d.getY())); 
        revalidate();
    }
    
    //--------------------------------
    // calcule du facteur de zoom pour voir toutes les villes dans la zone d'affichage
    public void calculerZoomVueDEnsemble() { 
	Point2D pt2d ;
    	if (tsp!=null) pt2d = tsp.getPoint2DMax() ;
    	else pt2d = new Point2D (500,500) ;
    	
    	double facteurZoomX=1, facteurZoomY=1 ;
    	if (pt2d.getX()>0) facteurZoomX = (double)jScrollPane.getSize().width/(double)pt2d.getX() ;
    	if (pt2d.getY()>0) facteurZoomY = (double)jScrollPane.getSize().height/(double)pt2d.getY() ;
    	facteurZoom = facteurZoomX ;
    	if (facteurZoomY<facteurZoom) facteurZoom = facteurZoomY ;

    	// Le viewport est mis a la taille du JScrollPane car le facteur de zoom va compresser les 
    	// coordonnees des points pour que le graphe entier soit visible dans le JScrollPane.
        setPreferredSize(new Dimension(jScrollPane.getSize().width, jScrollPane.getSize().height)); 
        revalidate();
    }
    
}


