//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.20
// Date modification : 08/01/2003
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import java.util.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Solveur generique du TSP (Voyageur de commerce)
public abstract class TspSolver {

  // renvoie 2 si il faut tout rafraichir
  public static final int RAFRAICHIR_TOUT = 2 ;

  // renvoie 1 si il faut rafraichir uniquement le numero d'iteration
  public static final int RAFRAICHIR_ITER = 1 ;

  // renvoie 0 sinon 
  public static final int RIEN            = 0 ;

  // renvoie -1 si il faut s'arreter
  public static final int STOP            = -1 ;  
  
  protected Tsp tsp ;
  Parcours solOpt ; // Meilleure solution obtenue

  protected double bestLongueurCycle=-1 ;
  protected int etatBestLongueurCycle = 0 ;
	
  protected long dateDebutResolution ;
  protected int init=1 ;
  protected long iter=0 ; // Iteration de recherche
  protected long iterAffichage=0, maxIterAffichage=100 ; // Iteration pour l'affichage
  protected Random rand ;
	
  //------------------------------
  public TspSolver (Tsp t) {
    solOpt = null ;
    dateDebutResolution = new Date().getTime() ;
    rand = new Random(dateDebutResolution) ;
    tsp = t ;
    init = 1 ;
  }
	
  public long getIter() { return iter ;}
  public Parcours getSolOpt() { return solOpt ; }
  public double getLongueurOpt() { 
    if (solOpt!=null) return solOpt.getLongueur() ; 
    else return -1 ;}
  public double getBestLongueurCycle() { return bestLongueurCycle ; }

  public void setSolOpt(Parcours p) { 
    solOpt = new Parcours(p) ; 
    tsp.solOpt = new Parcours(p) ; 
    if ((tsp.solOptTh==null) || (p.getLongueur()<tsp.solOptTh.getLongueur())) 
      tsp.solOptTh = new Parcours(p) ; 
  }

  public String getDuree() { 
    long dateCourante = new Date().getTime() ;

    long ms = dateCourante - dateDebutResolution ;

    long sec  = ms / 1000 ;
    ms -= sec * 1000 ;
		
    if (sec==0) return new String (ms+"ms") ;
		
    long min =  sec / 60 ;
    sec -= min * 60 ;
    if (min==0) return new String (sec + "sec " + ms+"ms") ;
		
    long heure = min / 60 ;
    min -= heure * 60 ;
    if (heure==0) return new String (min + "min " + sec + "sec") ;

    long jour = heure / 24 ;
    heure -= jour * 24 ;
    if (jour==0) return new String (heure + "hr " + min + "min") ;
    return new String (jour + "jr " + heure + "hr") ;
  }

  //------------------------------
  abstract public int run() ;
    	
}
