//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.31
// Date modification : 08/01/2003
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import java.util.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
class GeneParcours extends Parcours {
  int ind1, ind2 ; // indice de croisement

  //------------------------------
  GeneParcours () { 
    super () ;
  }

  //------------------------------
  GeneParcours (int n) { 
    super (n) ;
  }

  //------------------------------
  GeneParcours (GeneParcours p) { 
    super (p) ;
    ind1 = p.ind1 ;
    ind2 = p.ind2 ;
  }

  //------------------------------
  GeneParcours crossOver(Parcours parent2, int i1, int i2)
  {
		
    GeneParcours child = new GeneParcours(this) ;
    child.ind1 = this.ind1 ;
    child.ind2 = this.ind2 ;

    // on met les trous
    int i, k, j ;
    for (i = 0 ;i<nbrVilles;i++) {
      for (k=i1;k<=i2;k++) {
        if (child.elementAt(i)==parent2.elementAt(k))
          child.setElementAt (i, -1) ;
      }
    }
    i=i2+1 ;
    while ((i<i1) || (i>i2)) {
      if (  child.elementAt(i)<0 ) { // decalage
        j = i ;
        while (j!=i2) {
          if (j==nbrVilles-1) 
            child.setElementAt (j, child.elementAt(0)) ;
          else 
            child.setElementAt (j, child.elementAt(j+1)) ;
          j++ ;
          if (j>=nbrVilles) j=0 ;
        }
        child.setElementAt (i2, -1) ;
      }
      else {
        i++ ;
        if (i>=nbrVilles) i=0 ;
      }
    }
    for (k=i1;k<=i2;k++) {
      child.setElementAt ( k, parent2.elementAt(k)) ;
    }
    return child ;
  }

  //------------------------------
  void mutation (int ind1, int ind2)
  {
    int milieu = (ind2-ind1+1)/2 ;
    for (int i=0;i<milieu;i++) {
      int j = this.elementAt(ind1+i) ;
      this.setElementAt (ind1+i, this.elementAt(ind2-i)) ;
      this.setElementAt (ind2-i, j) ;
    }
  }
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
class Population extends Vector {
  private double dist[][] ;
  private int 	taille, // taille de la population
    nv,     // nbre de villes 
    maxindice ; 
  private double longmax, longmin ;
  private Parcours solOpt ; // Meilleure solution de la population
  private int coeffSelection = 1000 ; // l'augmenter pour garder les bons (minimum 1)
  private int tauxMutation = 5 ; // l'augmenter pour augmenter les mutations (entre 0 et 99)
  private Random rand ;

  //------------------------------
  Population (int taille, int n, double dist[][]) 
  {
    super() ;
    this.taille = taille ;
    this.nv = n ;
    this.dist = dist ;
  }
	
  public void setSolOpt (Parcours p) { solOpt = new Parcours(p) ; }
  public Parcours getSolOpt () { return solOpt ; }
  public double getLongueurOpt() { return solOpt.getLongueur() ; }
  public void setCoeffSelection (int c) { coeffSelection = c ; }
  public void setTauxMutation (int t) { tauxMutation = t ; }

  //------------------------------
  void genererAleat (Random rand) 
  {
    this.rand = rand ;
    for (int i=0;i<taille;i++) {
      GeneParcours p = new GeneParcours(nv) ;
      p.genereAleat(rand) ;
      p.calculeLongueur(dist) ;
      add (p) ;
    }
  }

  //------------------------------
  void preparerSelectionBons () 
  {
    longmax = 0 ;
    longmin = 0 ;
    taille = this.size() ;
    for (int i=0;i<taille;i++) {
      GeneParcours p = (GeneParcours)this.elementAt(i) ;
      if (p.getLongueur() > longmax) longmax = p.getLongueur() ;
      if ((i==0) || (p.getLongueur() < longmin)) {
        longmin = p.getLongueur() ;
        solOpt = p ;
      }
    }
    maxindice = 0 ;
    for (int i=0;i<taille;i++) {
      GeneParcours p = (GeneParcours)this.elementAt(i) ;
      p.ind1 = maxindice ;
      p.ind2 = maxindice + (int)(coeffSelection*(longmax-p.getLongueur())/(longmax-longmin)) ;
      maxindice=p.ind2+1 ;
    }
  }

  //------------------------------
  GeneParcours selectionBon (int ind) 
  {
    int i = 0 ;
    GeneParcours p = (GeneParcours)this.elementAt(0) ;
    while ((i<taille-1) && (ind>p.ind2)) {
      i++ ;
      p = (GeneParcours)this.elementAt(i) ;
    }
    return p ;
  }

  //------------------------------
  // A FAIRE
  void genererChilds () 
  {
    for (int i=0;i<taille/2;i++) {
      GeneParcours parent1 = selectionBon (rand.nextInt(maxindice)) ;
      GeneParcours parent2 = selectionBon (rand.nextInt(maxindice)) ;
      int i1=0, i2=0 ;
      while (i1>=i2) {
        i1 = rand.nextInt (nv-2)+1 ;
        i2 = rand.nextInt (nv-2)+1 ;
      }
      GeneParcours child1 = parent1.crossOver (parent2, i1, i2) ;
      GeneParcours child2 = parent2.crossOver (parent1, i1, i2) ;
      child1.calculeLongueur (dist) ;
      child2.calculeLongueur (dist) ;
      add (child1) ;
      add (child2) ;
      if (rand.nextInt(100)<=tauxMutation) { // Mutation
        child1.mutation (rand.nextInt (nv), rand.nextInt (nv)) ;
        child1.calculeLongueur (dist) ;
      }
      if (rand.nextInt(100)<=tauxMutation) { // Mutation
        child2.mutation (rand.nextInt (nv), rand.nextInt (nv)) ;
        child2.calculeLongueur (dist) ;
      }
    }
  }

  //------------------------------
  void preparerSelectionMauvais () 
  {
    longmax = 0 ;
    longmin = 0 ;
    for (int i=0;i<this.size();i++) {
      GeneParcours p = (GeneParcours)this.elementAt(i) ;
      if (p.getLongueur() > longmax) longmax = p.getLongueur() ;
      if ((i==0) || (p.getLongueur() < longmin)) {
        longmin = p.getLongueur() ;
        solOpt = p ;
      }
    }
    maxindice = 0 ;
    for (int i=0;i<this.size();i++) {
      GeneParcours p = (GeneParcours)this.elementAt(i) ;
      p.ind1 = maxindice ;
      p.ind2 = maxindice + (int)(coeffSelection*(p.getLongueur()-longmin)/(longmax-longmin)) ;
      maxindice=p.ind2+1 ;
    }
  }

  //------------------------------
  void supprimerMauvais () 
  {
    while (this.size()>taille) {
      int i = 0 ;
      int ind = rand.nextInt(maxindice) ;
      GeneParcours p = (GeneParcours)this.elementAt(0) ;
      while ((i<this.size()-1) && (ind>p.ind2)) {
        i++ ;
        p = (GeneParcours)this.elementAt(i) ;
      }
      this.removeElementAt (i) ;
    }
  }

}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Voyageur de commerce
public class TspSolverAG extends TspSolver {
  // AG
  int taillePop, nbrGene ;
  int coeffSelection, tauxMutation ;
  Population popX ;
	
  //------------------------------
  public TspSolverAG (Tsp t) {
    super (t) ;
    taillePop=nbrGene=-1 ;
    coeffSelection=tauxMutation=-1 ;
  }

  //------------------------------
  public void modifParam (String s1, String s2, String s3, String s4) {
    taillePop = Integer.parseInt (s1) ;
    nbrGene = Integer.parseInt (s2) ;
    coeffSelection = Integer.parseInt (s3) ;
    tauxMutation = Integer.parseInt (s4) ;
  }

  //------------------------------
  public void lireParametres1 () {
    // Initialise la population 
    System.out.print ("Taille de la population : ") ;
    taillePop = Clavier.lireEntier() ;
			
    System.out.print ("Nombre de generations : ") ;
    nbrGene = Clavier.lireEntier() ;
  }
	
  //------------------------------
  public void lireParametres2 () {
    System.out.print ("Coeff Selection - l augmenter pour garder les bons : ") ;
    popX.setCoeffSelection (Clavier.lireEntier()) ;
			
    System.out.print ("Pourcentage Mute - l augmenter pour avoir plus de mutation : ") ;
    popX.setTauxMutation (Clavier.lireEntier()) ;
  }
	
  //------------------------------
  // Effectue une iteration de recherche
  // renvoie RAFRAICHIR_TOUT si il faut tout rafraichir
  // renvoie RAFRAICHIR_ITER pour rafraichir uniquement le numero d'iteration
  // renvoie STOP si il faut s'arreter
  // renvoie RIEN sinon 
  public int run()
  {
    if (taillePop<0) return RIEN ;
    int nv = tsp.size() ;
    		
    if (init==1) {
      //    			lireParametres1() ;
      popX = new Population(taillePop, nv, tsp.getDist()) ;
      //	   		lireParametres2() ;
      popX.genererAleat(rand) ;
    }
    		
    iter++ ;
    if (iter>nbrGene) return STOP ;
    		
    iterAffichage += taillePop ;

    popX.preparerSelectionBons () ;
    popX.genererChilds () ;
    popX.preparerSelectionMauvais () ;
    popX.supprimerMauvais () ;
		
    if ((init==1) || (popX.getLongueurOpt()<getLongueurOpt())) {
      init = 0 ;
      setSolOpt (popX.getSolOpt()) ;
      return RAFRAICHIR_TOUT ;
    }

    if (iterAffichage>maxIterAffichage) {
      iterAffichage = 0 ;
      return RAFRAICHIR_ITER ;
    }
    return RIEN ;
  }

}
