//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.31
// Date modification : 08/01/2003
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import java.util.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
class Ant extends Parcours {
  int numero, villeDepart, villeCour, nbrVillesVisitees ;
  double[][] tabPhero ;
  double[] aij ;
  double alpha, beta ;

  //----------------------------------------
  Ant(int i, int nv, double[][] p, double[][] d, double a, double b) {
    super(nv, d) ;
    initAnt (i) ;
    tabPhero = p ;
    aij = new double[nbrVilles] ;    

    alpha = a ;
    beta = b ;
  }

  //----------------------------------------
  void initAnt(int i) {
    numero = i ;
    villeDepart = i ;
    villeCour = i ;
    nbrVillesVisitees = 1 ;
    longueur = -1 ;
    setElementAt (0, villeCour) ;
    for (int j=1;j<getNbrVilles();j++) setElementAt (j, -1) ;
  }

  //----------------------------------------
  int getNbrVillesVisitees() {return nbrVillesVisitees ;}

  //----------------------------------------
  boolean dejaVisitee(int v) {
    for (int i = 0;i<getNbrVilles();i++) {
      if (elementAt(i)==v) return true ;
    }
    return false ;
  }

  //----------------------------------------
  void afficheErreur(String mess) {
    System.out.println ("ERREUR ACO : "+mess+" pour numero "+numero) ;
    System.out.println ("nbrVillesVisitees "+getNbrVillesVisitees()) ;
    System.out.println ("VillesVisitees "+toString()) ;
    for (int j=0;j<nbrVilles;j++) {
      System.out.print ("aij["+j+"]="+aij[j]+" - ") ;
    }
    System.out.println() ;
    System.exit(0) ;
  }

  //----------------------------------------
  void affiche() {
    System.out.println ("VillesVisitees "+toString()) ;
  }

  //----------------------------------------
  void aff(String st) {
    System.out.println (st) ;
  }

  //----------------------------------------
  void run(Random rand) {
    double total = 0 ;

    for (int j=0;j<nbrVilles;j++) {
      if (j!=villeCour) {
        try {
          aij[j] = Math.pow(tabPhero[villeCour][j],alpha) ;
          aij[j] *= Math.pow(1/(dist[villeCour][j]+1),beta) ;
        } catch(Exception e) {
          afficheErreur ("index out of bounds") ;
        }
      }
      else aij[j] = 0 ;
      total += aij[j] ;
    }
    for (int j=0;j<nbrVilles;j++) {
      aij[j] /= total ;
    }

    total = 0 ;
    for (int j=0;j<nbrVilles;j++) {
      if (dejaVisitee(j)) aij[j] = 0 ;
      total += aij[j] ;
    }
    for (int j=0;j<nbrVilles;j++) {
      aij[j] /= total ;
    }

    int encore = 1 ;
    while (encore==1) {
      double tirage = rand.nextDouble() ;

      int j = 0 ;
      int choix = -1 ;
      total = 0 ;
      while ((j<nbrVilles) && (choix<0)) {
        total += aij[j] ;
        if (total>=tirage) choix = j ;
        else j++ ;
      }

      if (choix<0) {
        afficheErreur ("choix ngatif") ;
      }

      if (!dejaVisitee(choix)) {
        setElementAt(nbrVillesVisitees++, choix) ;
        encore = 0 ;
        villeCour = choix ;
      }
      else {
        System.out.println ("ERREUR Fourmi.run : i"+numero+" total"+total+" tirage"+tirage+" nbrVillesVisitees"+nbrVillesVisitees) ;
      }
    }
  }
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Voyageur de commerce
public class TspSolverAnt extends TspSolver {
  Ant tabAnt[] ;
  int nbrAnt ;
  long itmax ;
  double alpha, beta, ro ;
  double tabPhero[][] ;

  double long0 ;
	
  //------------------------------
  public TspSolverAnt (Tsp t) {
    super (t) ;
    alpha = 1 ;
    beta = 5 ;
    ro = 0.5 ;
    itmax = - 1 ;
    init =1 ; 
  }

  //------------------------------
  public void modifParam (String s1, String s2, String s3, String s4) {
    itmax = Long.parseLong(s1) ;
    alpha = Double.parseDouble (s2) ;
    beta = Double.parseDouble (s3) ;
    ro = Double.parseDouble (s4) ;
  }

  public static void aff (String st) {
    System.out.println(st) ;
  }

  //------------------------------
  // Effectue une iteration de recherche
  // renvoie RAFRAICHIR_TOUT si il faut tout rafraichir
  // renvoie RAFRAICHIR_ITER pour rafraichir uniquement le numero d'iteration
  // renvoie STOP si il faut s'arreter
  // renvoie RIEN sinon 
  public int run()
  {
    if (itmax<0) return RIEN ;

    int nv = tsp.size() ;
    int evap = 0 ;
    		
    if (init==1) {
      nbrAnt = nv ;
      tabAnt = new Ant[nbrAnt] ; 
      tabPhero = new double[nv][nv] ;
      for (int i=0;i<nv;i++)
        for (int j=0;j<nv;j++) 
          tabPhero[i][j]=0.1 ;
      for (int i=0;i<nbrAnt;i++) 
        tabAnt[i]=new Ant(i, nv, tabPhero, tsp.getDist(), alpha, beta) ;
      //      itmax = nbrAnt*nbrAnt*1000 ;
      init = 2 ;
      return RAFRAICHIR_TOUT ;
    }
    		
    iter++ ;
    if (iter>itmax) return STOP ;
    		
    iterAffichage += 1 ;

    int retour2 = 0 ;
    for (int i=0;i<nbrAnt;i++) {
      tabAnt[i].run(rand) ;
      if (tabAnt[i].getNbrVillesVisitees()>=nv) {
        double l = tabAnt[i].getLongueur() ;
        
        if (etatBestLongueurCycle==0) bestLongueurCycle = l ;
        else {
          etatBestLongueurCycle = 1 ;
          if (l<bestLongueurCycle) bestLongueurCycle = l ;
        }
        if (i==0) long0 = l ;
        if (init==2) { setSolOpt(tabAnt[i]) ; retour2 = 1 ; init = 0 ;
        }

        // A-t-on trouv une meilleure solution ?
        if (l<getLongueurOpt()) {
          setSolOpt (tabAnt[i]) ;
          retour2 = 1 ;
        }

        // Augmentation des phromones en fonction de la solution
        evap = 1 ;
        for (int m = 0 ; m <nv-1 ; m++) {
          try {
            int j = tabAnt[i].getVille(m) ;
            int k = tabAnt[i].getVille(m+1) ;
            tabPhero[j][k] += 1/(l*(1-ro)) ;
          } catch (Exception e) {
            System.out.println ("m="+m) ;
          }
        }
        int j = tabAnt[i].getVille(nv-1) ;
        int k = tabAnt[i].getVille(0) ;
        tabPhero[j][k] += 1/(l*(1-ro)) ;
        tabAnt[i].initAnt(i) ;
      }
    }
    etatBestLongueurCycle = 0 ;

    // Evaporation des phromones
    if (evap==1) {
      for (int i=0;i<nv;i++) 
        for (int j=0;j<nv;j++) 
          tabPhero[i][j] *= (1-ro) ;
    }


    if (retour2==1) return RAFRAICHIR_TOUT ;

    if (iterAffichage>maxIterAffichage) {
      iterAffichage = 0 ;
      return RAFRAICHIR_ITER ;
    }
    return RIEN ;
  }

}
