//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.12
// Date modification : 17/12/2002
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import java.util.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Solveur Little du TSP (Voyageur de commerce)
public class TspSolverLittle extends TspSolver {
  public static final boolean STANDARD = false ;
  public static final boolean AMELIORE = true ;
  private boolean testCycle ;

  //------------------------------
  public TspSolverLittle (Tsp t) {
    super (t) ;
    testCycle = true ;
  }

  //------------------------------
  public TspSolverLittle (Tsp t, boolean b) {
    super (t) ;
    testCycle = b ;
  }

  //------------------------------
  double[][] cloneMatrice(double[][] d) {
    int nlgn = d.length ;
    int ncol = d[0].length ;
    double[][] d2=new double[nlgn][ncol] ;
    for (int i=0;i<nlgn;i++) {
      for (int j=0;j<ncol;j++) {
        d2[i][j] = d[i][j] ;
      }
    }
    for (int i=0;i<nlgn;i++) 
      d2[i][i] = -1 ;
    return d2 ;
  }

  //------------------------------
  /** 
   * On soustrait le min de chaque ligne a la ligne correspondante
   * pour faire apparaitre au moins un zero par ligne
   * On fait de meme pour les colonnes
   * La somme des mins de ligne et des mins de colonne est renvoye
   * pour mettre  jour la valeur du sommet correspondant
   */
  public double miseAUnZeroParLigneEtColonne(double[][] d) {
    int nlgn = d.length ;
    int ncol = d[0].length ;
    double somme = 0 ;
    // Soustraction du min de chaque ligne
    for (int i=0;i<nlgn;i++) {
      double min = -1 ;
      for (int j=0;j<ncol;j++) 
        if ((d[i][j]>=0) && ((min<0) || (d[i][j]<min))) min = d[i][j] ;
      if (min>0) {
        for (int j=0;j<ncol;j++) {
          if (d[i][j]>=0)
            d[i][j] -= min ;
        }
        somme += min ;
      }
    }
    // Soustraction du min de chaque colonne
    for (int j=0;j<ncol;j++) {
      double min = -1 ;
      for (int i=0;i<nlgn;i++) 
        if ((d[i][j]>=0) && ((min<0) || (d[i][j]<min))) min = d[i][j] ;
      if (min>0) {
        for (int i=0;i<nlgn;i++) 
          if (d[i][j]>=0)
            d[i][j] -= min ;
        somme += min ;
      }
    }
    return somme ;
  }

  //------------------------------
  public int compteNombreZeros(double[][] d) {
    int nlgn = d.length ;
    int ncol = d[0].length ;
    int nz = 0 ;
    for (int i=0;i<nlgn;i++) {
      for (int j=0;j<ncol;j++) 
        if (d[i][j]==0) nz++ ;
    }
    return nz ;
  }

  //------------------------------
  public int calculePenalites(double[][] d, int[] lgnZero, int[] colZero, double[] penaliteZero) {
    int nlgn = d.length ;
    int ncol = d[0].length ;
    int nz = 0 ;
    double maxPenalite = 0 ;
    int indiceChoix = 0 ;
    for (int i=0;i<nlgn;i++) {
      for (int j=0;j<ncol;j++) 
        if (d[i][j]==0) {
          lgnZero[nz] = i ;
          colZero[nz] = j ;
          penaliteZero[nz] = 0 ;
          double min0 = -1 ;
          for (int i0=0;i0<nlgn;i0++) {
            if ((i!=i0) && (d[i0][j]>0)) {
              if ((min0<0) || (d[i0][j]<min0))
                min0 = d[i0][j] ;
            }
          }
          if (min0>0) penaliteZero[nz] += min0 ;
          min0 = -1 ;
          for (int j0=0;j0<ncol;j0++) {
            if ((j!=j0) && (d[i][j0]>0)){
              if ((min0<0) || (d[i][j0]<min0))
                min0 = d[i][j0] ;
            }
          }
          if (min0>0) penaliteZero[nz] += min0 ;
          if (penaliteZero[nz]>maxPenalite) {
            maxPenalite = penaliteZero[nz] ;
            indiceChoix = nz ;
          }
          nz ++ ;
        }
    }
    return indiceChoix ;
  }

  //------------------------------
  public void construireSolution(int[] listedepart, int[] listearrivee) {
    // On tient une solution
    Parcours p = new Parcours(tsp.size(), tsp.getDist()) ;
    boolean encore = true  ;
    int indiceCour = 0 ;
    int villeCour = 0 ;
    while (indiceCour<tsp.size()) {
      p.setElementAt(indiceCour, villeCour) ;

      // Test si le cycle est hamiltonien
      for (int i=0;i<indiceCour;i++) {
        if (p.elementAt(i)==villeCour) {
          //          System.out.println ("cycle non hamiltonien") ;
          return ;
        }
      }
      // Recherche de la ville suivante
      boolean trouve = false ;
      int i = 0 ;
      while ((!trouve) && (i<tsp.size())) {
        if (listedepart[i]==villeCour) {
          trouve = true ;
          villeCour = listearrivee[i] ;
        }
        i++ ;
      }
      indiceCour++ ;
    }
    p.calculeLongueur() ;
    //    System.out.println (p.getLongueur()) ;
    if (p.getLongueur()<getSolOpt().getLongueur()) {
      setSolOpt(p) ;
      System.out.println (p.toString()) ;
    }
    return  ;
  }

  //------------------------------
  public void solveLittle(double[][] d0, int iteration, double evalNoeudParent, int[] listedepart, int[] listearrivee) {
    if (iteration==tsp.size()) {
      //      System.out.print ("Construction d'une solution ("+evalNoeudParent+") : ") ;
      construireSolution (listedepart, listearrivee) ;
      return ;
    }
    else { // Vrification qu'il n'y a pas de cycle dans solution partielle...
      if (testCycle && (iteration>1)) {
        int indiceCour = iteration-1 ;
        boolean encore = true ;
        while (encore) {
          int i=0 ;
          boolean trouve = false ;
          while ((!trouve) && (i<iteration-1)) {
            if (listedepart[i]==listearrivee[indiceCour]) {
              trouve = true ;
              indiceCour = i ;
            }
            i++ ;
          }
          if (listearrivee[indiceCour]==listedepart[iteration-1]) // Cycle
            return ;
          encore = trouve ;
        }
      }
    }

    int nlgn = d0.length ;
    int ncol = d0[0].length ;
    double evalNoeudCourant = evalNoeudParent ;
    double[][] d = cloneMatrice(d0) ;

    // Mise  1 zero par ligne et par colonne
    evalNoeudCourant += miseAUnZeroParLigneEtColonne(d) ;

    // Est-ce qu'on peut faire mieux que l'optimal ?
    if (evalNoeudCourant>=getSolOpt().getLongueur()) return ;

    // Calcul des pnalits et choix de la plus forte pnalit
    int nbrZeros = compteNombreZeros(d) ;
    if (nbrZeros==0) return ;
    
    int[] lgnZero = new int[nbrZeros] ;
    int[] colZero = new int[nbrZeros] ;
    double[] penaliteZero = new double[nbrZeros] ;
    int indiceChoix = calculePenalites(d, lgnZero, colZero, penaliteZero) ;
    int lgnChoix = lgnZero[indiceChoix] ;
    int colChoix = colZero[indiceChoix] ;

    listedepart[iteration] = lgnChoix ;
    listearrivee[iteration] = colChoix ;

    lgnZero = null ;
    colZero = null ;

    double[][] d1 = cloneMatrice(d) ;
    for (int i=0;i<nlgn;i++) 
      d1[i][colChoix] = -1 ;
    for (int j=0;j<ncol;j++) 
      d1[lgnChoix][j] = -1 ;
    d1[colChoix][lgnChoix] = -1 ;
    solveLittle (d1, iteration+1, evalNoeudCourant, listedepart, listearrivee) ;

    d1 = null ;

    d[lgnChoix][colChoix] = -1 ;
    solveLittle (d, iteration, evalNoeudCourant, listedepart, listearrivee) ;
  }

  //------------------------------
  // Effectue une iteration de recherche
  // renvoie RAFRAICHIR_TOUT si il faut tout rafraichir
  // renvoie RAFRAICHIR_ITER pour rafraichir uniquement le numero d'iteration
  // renvoie STOP si il faut s'arreter
  // renvoie RIEN sinon 
  public int run()
  {
    iter++ ;
    switch (init) {
    case 1 :
      init++ ;
      Parcours p = new Parcours (tsp.size(), tsp.getDist()) ;
    
      // Choix d'une solution initiale obtenue par algo Glouton
      //      solutionCourante.genererAleat(rand) ;
      p.genereGlouton() ;
      p.calculeLongueur() ;
      setSolOpt (p) ;
      return RAFRAICHIR_TOUT ;
    case 2 :
      init = 0 ;
      int[] listedepart = new int[tsp.size()];
      int[] listearrivee = new int[tsp.size()];
      System.out.println ("Beginning of optimal search with Little Algorithm...") ;
      solveLittle(tsp.getDist(), 0, 0, listedepart, listearrivee) ;
      System.out.println ("End of optimal search with Little Algorithm !!!") ;
      System.out.println ("Time = "+getDuree()) ;
      return RAFRAICHIR_TOUT ;
    }
    return STOP ;
  }
	
}
