//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.12
// Date modification : 17/12/2002
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import java.util.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Solveur Recuit du TSP (Voyageur de commerce)
public class TspSolverRecuit extends TspSolver {

  private Parcours solutionCourante ;

  private double temp ; // temperature 
  private double alpha ; // coefficent d'evolution de la temperature 
  private int iterTemp, iterTempMax ;

  //------------------------------
  public TspSolverRecuit (Tsp t) {
    super (t) ;
    temp = -1 ;
    alpha = 0.9 ;
    iterTemp = 0 ;
    iterTempMax = 10000 ;
    //    maxIterAffichage = 10000 ;
  }

  //------------------------------
  public void modifParam (String s1, String s2, String s3) {
    temp = Double.parseDouble (s1) ;
    alpha = Double.parseDouble (s2) ;
    iterTempMax = Integer.parseInt (s3) ;
  }

  //------------------------------
  // Effectue une iteration de recherche
  // renvoie RAFRAICHIR_TOUT si il faut tout rafraichir
  // renvoie RAFRAICHIR_ITER pour rafraichir uniquement le numero d'iteration
  // renvoie STOP si il faut s'arreter
  // renvoie RIEN sinon 
  public int run()
  {
    if (temp<0) return RIEN ;
    iter++ ;
    if (init==1) {
      init = 0 ;
      solutionCourante = new Parcours (tsp.size(), tsp.getDist()) ;

      // Choix d'une solution initiale obtenue par algo Glouton
      //      solutionCourante.genererAleat(rand) ;
      solutionCourante.genereGlouton() ;
      solutionCourante.calculeLongueur() ;
      setSolOpt (solutionCourante) ;

      return RAFRAICHIR_TOUT ;
    }
    else {
      iterAffichage++ ;

      // On tire tsp.size() Voisins en 2-opt
      for (int i=0;i<tsp.size();i++) {
        Parcours solutionSuivante = solutionCourante.genereVoisin2Opt(rand) ;
        if (solutionSuivante.calculeLongueur()<=solutionCourante.getLongueur()) {
          solutionCourante = solutionSuivante ;
          if (solutionCourante.getLongueur()<getSolOpt().getLongueur()) {
            setSolOpt (solutionCourante) ;
          }
          return RAFRAICHIR_TOUT ;
        }
        else {
          double deltaF = solutionSuivante.getLongueur()-solutionCourante.getLongueur() ;
          double p = Math.exp (-deltaF/temp) ;
          double q = rand.nextDouble() ;
          if (q<=p) {
            solutionCourante = solutionSuivante ;
          }
        }

        iterTemp++ ;
        if (iterTemp>iterTempMax) { // reduction de la temperature
          iterTemp = 0 ;
          temp *= alpha ;
        }
      }
    }
    if (iterAffichage>maxIterAffichage) {
      iterAffichage = 0 ;
      return RAFRAICHIR_ITER ;
    }
    return RIEN ;
  }
	
}
