//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Projet : gtsp (voyageur de commerce)
// Date debut projet : 16/12/2002
// Auteur : Olivier Grunder
//
// Version : 0.12
// Date modification : 17/12/2002
// Auteur : Olivier Grunder
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import java.util.* ;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// Solveur Tabou du TSP (Voyageur de commerce)
public class TspSolverTabou extends TspSolver {

    public class Permutation {
	int ind1, ind2 ;
    }

    private Parcours solutionCourante ;
    private Parcours solutionMeilleure ;
    private int tailleListeTabou ;
    private double coeffAspiration ;
    private int indiceListeTabou ;
    private Permutation[] listeTabouInd ;
    private int iterationSansAmelioration;

    //------------------------------
    public TspSolverTabou (Tsp t) {
	super (t) ;
	tailleListeTabou = -1 ;
	indiceListeTabou = 0 ;
	iterationSansAmelioration = 0;
    }

    //------------------------------
    public void modifParam (String s1, String s2) {
	tailleListeTabou = Integer.parseInt (s1) ;
	coeffAspiration = Double.parseDouble (s2) ;

	indiceListeTabou = 0 ;
	listeTabouInd = new Permutation[tailleListeTabou];
    }

    private void ajouteListeTabou(int ind1, int ind2)
    {
	if(listeTabouInd[indiceListeTabou] == null)
	    {
		listeTabouInd[indiceListeTabou] = new Permutation();
	    }
	
	listeTabouInd[indiceListeTabou].ind1 = ind1;
	listeTabouInd[indiceListeTabou].ind2 = ind2;

	indiceListeTabou++ ;
	
	if (indiceListeTabou>=tailleListeTabou)
	    indiceListeTabou = 0 ;
    }

    private boolean estDansListeTabou(int ind1, int ind2)
    {
	for (int i = 0; i < tailleListeTabou; i++)
	    {
		if((listeTabouInd[i] != null) &&
		   (listeTabouInd[i].ind1 == ind1) &&
		   (listeTabouInd[i].ind2 == ind2))
		    {
			return true;
		    }
	    }

	return false;
    }

    private Parcours trouveMeilleurMouvement3(Parcours solutionCourante, int taille)
    {
	double longueur = 0;
	Parcours meilleurParcours = null;
	int ind1 = -1, ind2 = -1;

        for(int i = 1; i < taille; i++)
	    {
		for(int j = -5; j <= 5; j++ )
		    {
			if( (i+j >= 0) && (i+j < taille) && (j != 0) )
			    {
				Parcours p = new Parcours (solutionCourante) ;
				int temp = p.elementAt(i) ;

				if(estDansListeTabou(i, j) &&
				   Math.random() < coeffAspiration)
				    {
					continue;
				    }
				
				p.setElementAt (i, p.elementAt(i+j)) ;
				p.setElementAt (i+j, temp) ;

				p.calculeLongueur();

				if((longueur == 0) ||
				   (p.getLongueur() < longueur))
				    {
					meilleurParcours = p;
					longueur = p.getLongueur();
					ind1 = i;
					ind2 = j;
				    }
				
			    }
		    }
	    }
	
	if(meilleurParcours != null)
	    {
		ajouteListeTabou(ind1, ind2);
	    }

	return meilleurParcours;
    }

    private Parcours trouveMeilleurMouvement2(Parcours solutionCourante, int taille)
    {
	double longueur = 0;
	Parcours meilleurParcours = null;
	int customer = 0;
	int ind1 = -1, ind2 = -1;

	for (int i = 0 ; i < 50 ; i++)
	    {
		int indice1 = (int) (Math.random() * taille);
		int indice2 = (int) (Math.random() * taille);

		if(indice1 == indice2)
		    {
			i--;
			continue;
		    }

		Parcours p = new Parcours (solutionCourante) ;
		int temp = p.elementAt(indice1) ;

		if(estDansListeTabou(indice1, indice2) &&
		   Math.random() < coeffAspiration)
		    {
			continue;
		    }

		p.setElementAt (indice1, p.elementAt(indice2)) ;
		p.setElementAt (indice2, temp) ;

		p.calculeLongueur();

		if((longueur == 0) ||
		   (p.getLongueur() < longueur))
		    {
			meilleurParcours = p;
			longueur = p.getLongueur();
			ind1 = indice1;
			ind2 = indice2;
		    }
	    }

	if(meilleurParcours != null)
	    {
		ajouteListeTabou(ind1, ind2);
	    }

	return meilleurParcours;
    }

    private Parcours trouveMeilleurMouvement(Parcours solutionCourante, int taille)
    {
	double longueur = 0;
	Parcours meilleurParcours = null;
	int ind1 = -1, ind2 = -1;

	for (int i = 0 ; i < taille ; i++)
	    {
		int j = i + 1;
		if (j >= taille)
		    {
			j -= taille;
		    }

		Parcours p = new Parcours (solutionCourante) ;
		int temp = p.elementAt(i) ;

		if(estDansListeTabou(i, j) &&
		   Math.random() < coeffAspiration)
		    {
			continue;
		    }

		p.setElementAt (i, p.elementAt(j)) ;
		p.setElementAt (j, temp) ;

		p.calculeLongueur();

		if((longueur == 0) ||
		   (p.getLongueur() < longueur))
		    {
			meilleurParcours = p;
			longueur = p.getLongueur();
			ind1 = i;
			ind2 = j;
		    }
	    }

	if(meilleurParcours != null)
	    {
		ajouteListeTabou(ind1, ind2);
	    }

	return meilleurParcours;
    }

    public double meilleureLongueur()
    {
	return solutionMeilleure.calculeLongueur();
    }

    public double couranteLongueur()
    {
	return solutionCourante.calculeLongueur();
    }

    //------------------------------
    // Effectue une iteration de recherche
    // renvoie RAFRAICHIR_TOUT si il faut tout rafraichir
    // renvoie RAFRAICHIR_ITER pour rafraichir uniquement le numero d'iteration
    // renvoie STOP si il faut s'arreter
    // renvoie RIEN sinon 
    public int run()
    {
	if (tailleListeTabou<0) return RIEN ;
	iter++ ;

	/* Initialisation (premire itration) */
	if (init==1) {
	    init = 0 ;
	    solutionCourante = new Parcours (tsp.size(), tsp.getDist()) ;

	    // Choix d'une solution initiale obtenue par algo Glouton
	    //solutionCourante.genereAleat(rand) ;
	    solutionCourante.genereGlouton() ;
	    solutionCourante.calculeLongueur() ;
	    setSolOpt (solutionCourante) ;

	    solutionMeilleure = solutionCourante;

	    return RAFRAICHIR_TOUT ;
	}
	else {
	    iterAffichage++ ;

	    double minlongueur = -1 ;
	    int minind = -1 ;
	    int customer = -1;

	    solutionCourante = trouveMeilleurMouvement2(solutionCourante, tsp.size());

	    if(solutionCourante.calculeLongueur() < solutionMeilleure.calculeLongueur())
		{
		    setSolOpt(solutionCourante);
		    solutionMeilleure = solutionCourante;
		    return RAFRAICHIR_TOUT;
		}
	}

	if (iterAffichage>maxIterAffichage) 
	    {
		iterAffichage = 0 ;
		return RAFRAICHIR_ITER ;
	    }

	return RIEN ;
    }	
}
